from typing import Any

import click
from requests import HTTPError

from mmemoji import Emoji
from mmemoji.decorators import parse_global_options


@click.command(help="List custom Emojis")
@parse_global_options
def cli(ctx: Any) -> None:
    try:
        ctx.print_dict(Emoji.list(ctx.mattermost))
    except HTTPError as e:
        raise click.ClickException(e.args[0] if e.args != () else repr(e))



from typing import Any, List

import click
from requests import HTTPError

from mmemoji import Emoji
from mmemoji.decorators import parse_global_options


@click.command(help="Create custom Emojis")
@click.argument("images", type=click.File("rb", lazy=True), nargs=-1)
@click.option(
    "-f",
    "--force",
    is_flag=True,
    help="""
if the emoji exists, remove it and proceed \
(this option is ignored when the -i or -n option is also used)
""",
)
@click.option(
    "-n",
    "--no-clobber",
    is_flag=True,
    help="do not overwrite an existing file (overrides -i option)",
)
@click.option(
    "-i", "--interactive", is_flag=True, help="prompt before overwrite"
)
@parse_global_options
def cli(
    ctx: Any,
    images: List[str],
    force: bool,
    no_clobber: bool,
    interactive: bool,
) -> None:
    emojis = []

    try:
        with click.progressbar(images, show_pos=True) as pb_images:
            for image in pb_images:
                emoji = Emoji(ctx.mattermost, image.name)

                if emoji.metadata and not no_clobber and interactive:
                    force = click.confirm(
                        'overwrite "{}"?'.format(emoji.name), err=True
                    )
                    if not force:
                        continue

                with image as img:
                    if emoji.create(img, force, no_clobber):
                        emojis.append(emoji.metadata)
    except HTTPError as e:
        raise click.ClickException(e.args[0] if e.args != () else repr(e))
    finally:
        ctx.print_dict(emojis)



import imghdr
import os
from typing import Any, List

import click
from requests import HTTPError

from mmemoji import Emoji
from mmemoji.decorators import parse_global_options


def check_destination(ctx: Any, param: click.Parameter, value: str) -> str:
    """Ensure destination is valid.

    For 1 emoji, the destination can be a file
    with an existing parent directory, or an existing directory,
    but if the destination is explicitly a directory,
    the full path must exits.

    For more than 1 emoji, the destination has to be an existing directory.

    Finally, the destination has to be writable.
    """
    count = len(ctx.params["emoji_names"])
    if count == 1 and value[-1] != os.path.sep and not os.path.isdir(value):
        directory = os.path.dirname(value) or os.getcwd()
    else:
        directory = value
    if not os.path.isdir(directory):
        raise click.ClickException("{}: Not a directory".format(directory))
    if (
        not os.access(directory, os.W_OK)
        or os.path.isfile(value)
        and not os.access(value, os.W_OK)
    ):
        raise click.ClickException("{}: Permission denied".format(directory))
    return value


@click.command(help="Download custom Emojis")
@click.argument("emoji_names", nargs=-1)
@click.argument("destination", callback=check_destination, type=click.Path())
@click.option(
    "-f",
    "--force",
    is_flag=True,
    help="do not prompt before overwriting (overrides -i option)",
)
@click.option(
    "-n",
    "--no-clobber",
    is_flag=True,
    help="do not overwrite an existing file (overrides -f and -i options)",
)
@click.option(
    "-i", "--interactive", is_flag=True, help="prompt before overwrite"
)
@parse_global_options
def cli(
    ctx: Any,
    emoji_names: List[str],
    destination: str,
    force: bool,
    no_clobber: bool,
    interactive: bool,
) -> None:
    try:
        for name in emoji_names:
            emoji = Emoji(ctx.mattermost, name)
            image = emoji.download()

            if not os.path.isdir(destination):
                filename = destination
            else:
                filename = os.path.join(
                    destination, "{}.{}".format(name, imghdr.what(None, image))
                )

            if os.path.exists(filename) and (
                not interactive
                and (no_clobber or not force)
                or interactive
                and not click.confirm(
                    'overwrite "{}"?'.format(filename), err=True
                )
            ):
                continue

            with open(filename, "wb") as f:
                f.write(image)
            click.echo(filename)
    except HTTPError as e:
        raise click.ClickException(e.args[0] if e.args != () else repr(e))



import sys

from mmemoji.emoji import Emoji

# https://github.com/python/mypy/issues/1153
if sys.version_info[:2] >= (3, 8):
    import importlib.metadata as importlib_metadata
else:
    import importlib_metadata


__all__ = ["Emoji"]
__version__ = importlib_metadata.version(__name__)
__summary__ = importlib_metadata.metadata(__name__)["Summary"]



from typing import Any, List

import click
from requests import HTTPError

from mmemoji import Emoji
from mmemoji.decorators import parse_global_options


@click.command(help="Delete custom Emojis")
@click.argument("emoji_names", nargs=-1)
@click.option("-f", "--force", is_flag=True, help="ignore nonexistent files")
@click.option(
    "-i", "--interactive", is_flag=True, help="prompt before every removal"
)
@parse_global_options
def cli(
    ctx: Any, emoji_names: List[str], force: bool, interactive: bool
) -> None:
    emojis = []
    try:
        with click.progressbar(emoji_names, show_pos=True) as pb_names:
            for name in pb_names:
                emoji = Emoji(ctx.mattermost, name)

                if (
                    interactive
                    and emoji.metadata
                    and not click.confirm(
                        'delete "{}"?'.format(emoji.name), err=True
                    )
                ):
                    continue

                if emoji.delete(force):
                    emojis.append(emoji.metadata)
    except HTTPError as e:
        raise click.ClickException(e.args[0] if e.args != () else repr(e))
    finally:
        ctx.print_dict(emojis)



"""Wrapper for Mattermost API ``/emoji`` Endpoint.

This wrapper is built around ``python-mattermostdriver``_

.. _python-mattermostdriver:
    https://vaelor.github.io/python-mattermost-driver/
"""

import re
from os.path import basename
from typing import IO, Any, Dict, List, cast

from mattermostdriver.exceptions import ResourceNotFound
from unidecode import unidecode

from mmemoji.exceptions import EmojiAlreadyExists, EmojiNotFound


class Emoji:
    """Interact with Mattermost custom Emojis."""

    def __init__(self, mattermost: Any, name: str) -> None:
        """Init Emoji class with a Mattermost client instance and an Emoji name.

        Parameters
        ----------
        mattermost : :obj:`mattermostdriver.Driver`
            an instance of `mattermostdriver`_
        name : str
            an Emoji name. It can be a file path,
            the filename will be automatically extracted and sanitized
        """
        self._mm = mattermost
        self._name = self.sanitize_name(name)
        self._metadata: Dict[str, Any] = {}

    @staticmethod
    def sanitize_name(filepath: str) -> str:
        """Extract and sanitize an Emoji name from a file path.

        Parameters
        ----------
        filepath : str
            Emoji file path (e.g. ``/path/to/emoji (1).gif``)

        Returns
        -------
        str
            Emoji name (e.g. ``emoji_1``)
        """
        # Extract filename without extension
        name = basename(filepath).split(".")[0]
        # Transliterate Unicode to ASCII (remove accents)
        name = unidecode(name)
        # Remove parentheses
        name = re.sub(r"[()[\]{}]", "", name)
        # Replace forbidden characters by underscores
        name = re.sub(r"[^a-zA-Z0-9_-]", "_", name)
        return name

    def _get_metadata_from_mattermost(self) -> bool:
        """Retrieve custom Emoji metadata from Mattermost."""
        try:
            self._metadata = self._mm.emoji.get_custom_emoji_by_name(self.name)
            return True
        except ResourceNotFound:
            self._metadata = {}
            return False

    @property
    def metadata(self) -> Dict[str, Any]:
        """:obj:`dict` of (str: Any): Gets Emoji metadata."""
        if not self._metadata:
            self._get_metadata_from_mattermost()
        return self._metadata

    @property
    def name(self) -> str:
        """str: Get Emoji name."""
        return self._name

    def create(
        self, image: IO[bytes], force: bool = False, no_clobber: bool = False
    ) -> bool:
        """Create a custom Emoji on Mattermost.

        Parameters
        ----------
        image : :obj:`file`
            an image to upload
        force: bool
            delete Emoji if it already exits
            (ignored if ``no_clobber is ``True``)
        no_clobber: bool
            do nothing if Emoji already exits

        Returns
        -------
        bool
            Returns ``True`` if Emoji was created

        Raises
        ------
        EmojiAlreadyExists
            If nor ``no_clobber`` or ``force`` were ``True``
        """
        if self.metadata:
            if no_clobber:
                return False
            elif force:
                self.delete()
            else:
                raise EmojiAlreadyExists(self)

        self._metadata = self._mm.emoji.create_custom_emoji(
            emoji_name=self._name, files={"image": image}
        )
        return True

    def delete(self, force: bool = False) -> bool:
        """Delete a custom Emoji on Mattermost.

        Parameters
        ----------
        force: bool
            Ignore non-existent Emoji

        Returns
        -------
        bool
            Returns ``True`` if Emoji was deleted

        Raises
        ------
        EmojiNotFound
            If Emoji does not exist and ``force`` was not ``True``
        """
        if not self.metadata and force:
            return False

        if self.metadata:
            self._mm.emoji.delete_custom_emoji(self.metadata.get("id", ""))
            return True
        else:
            raise EmojiNotFound(self)

    @staticmethod
    def list(
        mattermost: Any, page: int = 0, per_page: int = 200, sort: str = "name"
    ) -> List[Dict[str, Any]]:
        """List custom Emojis on Mattermost.

        Parameters
        ----------
        mattermost : :obj:`mattermostdriver.Driver`
            an instance of `mattermostdriver`_
        page: int
            The page to select.
        per_page: int
            The number of users per page.
        sort: string
            Either blank for no sorting or "name" to sort by emoji names.

        Returns
        -------
        :obj:`list` of `dict`
            Returns a list of Emoji metadata
        """
        metadata_list = []
        count, previous_count = 0, 0
        params = cast(
            Dict[str, Any],
            {"page": page, "per_page": per_page, "sort": sort},
        )
        while True:
            metadata_list += mattermost.emoji.get_emoji_list(params=params)
            count = len(metadata_list)
            if count - previous_count < per_page:
                break
            # https://github.com/python/mypy/issues/3816
            params["page"] += 1
            previous_count = count
        return metadata_list

    @staticmethod
    def search(
        mattermost: Any, term: str, prefix_only: bool = False
    ) -> List[Dict[str, Any]]:
        """Search custom Emojis on Mattermost.

        Parameters
        ----------
        mattermost : :obj:`mattermostdriver.Driver`
            an instance of `mattermostdriver`_
        term: str
            The term to match against the emoji name.
        prefix_only: bool
            Set to only search for names starting with the search term.

        Returns
        -------
        :obj:`list` of `dict`
            Returns a list of Emoji metadata
        """
        return cast(
            List[Dict[str, Any]],
            mattermost.emoji.search_custom_emoji(
                options={"term": term, "prefix_only": prefix_only}
            ),
        )

    def download(self) -> bytes:
        """Download a custom Emoji from Mattermost.

        Returns
        -------
        bytes
            Returns Emoji image

        Raises
        ------
        EmojiNotFound
            If Emoji does not exist
        """
        if self.metadata and "id" in self.metadata:
            return cast(
                bytes,
                self._mm.emoji.get_custom_emoji_image(
                    self.metadata["id"]
                ).content,
            )
        raise EmojiNotFound(self)



import logging
import os
from typing import Any, List, cast

import click

from mmemoji import __summary__, __version__

logging.getLogger("mattermostdriver.websocket").disabled = True


class EmojiCLI(click.MultiCommand):
    """Custom Click Command class to dynamically discover subcommands"""

    def list_commands(self, ctx: Any) -> List[str]:
        rv = []
        cmd_folder = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "commands")
        )
        for filename in os.listdir(cmd_folder):
            if filename.endswith(".py") and not filename == "__init__.py":
                rv.append(filename[:-3])
        rv.sort()
        return rv

    def get_command(self, ctx: Any, name: str) -> click.Command:
        try:
            module = __import__(
                "mmemoji.commands." + name, None, None, ["cli"]
            )
            return cast(click.Command, module.cli)
        except ModuleNotFoundError:
            raise click.ClickException(
                'Unknown command "{}" for "{}"\n'
                "Run '{} --help' for usage.".format(
                    name, ctx.info_name, ctx.info_name
                )
            )


@click.command(name="mmemoji", cls=EmojiCLI, help=__summary__)
@click.version_option(version=__version__, message="%(prog)s %(version)s")
def cli() -> None:
    """CLI entry-point"""
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mmemoji', 'mmemoji.commands']

package_data = \
{'': ['*']}

install_requires = \
['Unidecode',
 'click>=7.0',
 'mattermostdriver>=6.1.2',
 'requests',
 'tablib',
 'tabulate']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata']}

entry_points = \
{'console_scripts': ['mmemoji = mmemoji.cli:cli']}

setup_kwargs = {
    'name': 'mmemoji',
    'version': '0.3.0',
    'description': 'Custom Emoji manager command-line for Mattermost 😎',
    'long_description': "[![PyPI][pypi badge]][pypi link]\n[![Build Status][build badge]][build link]\n[![Quality Gate][sonarcloud badge]][sonarcloud link]\n\n# mmemoji\n\nCustom Emoji manager command-line for [Mattermost][mattermost] 😎\n\nFeatures:\n\n* Create custom Emojis\n* Delete custom Emojis\n* List custom Emojis\n* Search custom Emojis\n* Export custom Emojis\n\n## Installation\n\n\n```shell\npip install mmemoji\nmmemoji --help\n```\n\n_(Requires Python >=3.5)_\n\n## Usage example\n\nLet's take the [Party Parrot][COTPP] Emojis as an example.\n\n* First, clone the Git repository or retrieve an archive of it:\n\n```shell\ngit clone https://github.com/jmhobbs/cultofthepartyparrot.com.git\ncd cultofthepartyparrot.com\n```\n\n* Then you'll need your Mattermost credentials. You can either pass them to `mmemoji` with the arguments `--url`/`--login-id`/`--password` or via environment variables, for example:\n\n```shell\nexport MM_URL='http://127.0.0.1:8065/api/v4'\nexport MM_LOGIN_ID='user-1@sample.mattermost.com'\nexport MM_PASSWORD='user-1'\n```\n\n* Finally, run `mmemoji` to import all the parrots:\n\n```shell\nmmemoji create --no-clobber {parrots,guests}/hd/*.gif {parrots,guests}/*.gif\n```\n\n> _Notes_:\n>\n> * Here we rely on [shell globbing][glob] to select all emojis from the directories.\n> * Specifying the `hd` directories first with `--no-clobber` ensures these emojis are created first and not overwritten by their lower quality counterpart.\n\n* If you ever want to remove them all, simply run the following:\n\n```shell\nmmemoji delete --force {parrots,guests}/hd/*.gif {parrots,guests}/*.gif\n```\n\n> _Notes_:\n>\n> * The emoji names are extracted from the filenames the same way they have been during creation.\n> * `--force` is used to ignore the absent low quality duplicates.\n\n## Development\n\n* You can clone this repository and install the project with [Poetry][poetry]:\n\n```shell\npoetry install\n```\n\n* You'll find a script to create a local [Docker][docker] test instance under `tests/`:\n\n```shell\n./tests/scripts/setup-mattermost.sh\n```\n\n* You can run the test suite with:\n\n```shell\npytest\n```\n\n* And last thing, you can install the [pre-commit][pre-commit] hooks to help with the formatting of your code.\n\n```shell\npre-commit install\n```\n\n[pypi badge]: https://img.shields.io/pypi/v/mmemoji.svg\n[pypi link]: https://pypi.python.org/pypi/mmemoji\n[build badge]: https://github.com/maxbrunet/mmemoji/actions/workflows/build.yml/badge.svg\n[build link]: https://github.com/maxbrunet/mmemoji/actions/workflows/build.yml\n[sonarcloud badge]: https://sonarcloud.io/api/project_badges/measure?project=maxbrunet_mmemoji&metric=alert_status\n[sonarcloud link]: https://sonarcloud.io/dashboard?id=maxbrunet_mmemoji\n[mattermost]: https://www.mattermost.org\n[COTPP]: https://cultofthepartyparrot.com\n[glob]: https://en.wikipedia.org/wiki/Glob_(programming)\n[poetry]: https://python-poetry.org/docs/\n[docker]: https://www.docker.com\n[pre-commit]: https://pre-commit.com\n",
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/maxbrunet/mmemoji.git',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)
